package lamda.optional;

import java.util.Optional;
import java.util.function.Function;
import java.util.function.Predicate;

//It is used to represent a value is present or not
//a way to deal with null values
public class OptionalExamples {
    public static void main(String[] args) {

        Optional<String> empty = Optional.empty();
        Optional<String> nonEmpty = Optional.of("FIFA Worldcup 2022");

        //check empty
        Optional.ofNullable("hyderabad").ifPresent(System.out::println); //hyderabad
        System.out.println(empty.isPresent()); //false
        System.out.println(nonEmpty.isPresent()); //true

        //Check Null
        Optional<Object> nullCheck = Optional.ofNullable(null);
        Optional<String> nullCheck1 = Optional.of(null);
        System.out.println("is ofNullable "+nullCheck);//
        System.out.println("is Null "+nullCheck1.get());// NPE

        //filter(Predicate)
        Predicate<String> year = s->s.contains("2022");

        System.out.println(nonEmpty.filter(year));//Optional[FIFA Worldcup 2022]
        System.out.println(empty.filter(year));// Optional.empty
        System.out.println(nonEmpty.filter(x->x.startsWith("FIFA")).get());

        //map(Function)
        Function<String, String> func = x-> x.substring(5,13);
        System.out.println(nonEmpty.map(func).get()); //Worldcup
        System.out.println(nonEmpty.map(x->x.toLowerCase()).get()); //fifa worldcup 2022

        //flatmap(Function)
        Optional<Optional<String>> opopS  = Optional.of(Optional.of("FIFA Worldcup 2022"));

        Function<String, String> func1 = x->x.concat(" Qatar");
        System.out.println(opopS.map(x->x.map(func1)).get());//Optional[FIFA Worldcup 2022 Qatar]
        System.out.println(opopS.flatMap(x->x.map(func1)).get());//FIFA Worldcup 2022 Qatar

        //orElse
        System.out.println(nonEmpty.orElse("Next Cup")); //FIFA Worldcup 2022
        System.out.println(empty.orElse("Next Cup")); // Next Cup

        //exception
        System.out.println(nonEmpty.orElseThrow(NullPointerException::new)); //FIFA Worldcup 2022
        System.out.println(empty.orElseThrow(NullPointerException::new));//Exception in thread "main" java.lang.NullPointerException


    }
}
