package stream.basic;

import stream.basic.pojo.Employee;

import java.util.*;
import java.util.function.BinaryOperator;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class CollectorsDemo2 {

    public static void main(String[] args) {

        List<Employee> employeeList = new ArrayList<>();
        employeeList.add(new Employee(100, "gautam", 47, "North America", 450000));
        employeeList.add(new Employee(200, "adani", 25, "North America", 50000));
        employeeList.add(new Employee(300, "mukesh", 30, "Asia", 450000));
        employeeList.add(new Employee(400, "ambani", 59, "Africa", 450000));

        List<String> names = employeeList.stream().map(e -> e.getName()).collect(Collectors.toList());
        System.out.println(names); //[gautam, adani, mukesh, ambani]

        //summingInt :  summing all salaries
        Double collect = employeeList.stream().collect(Collectors.summingDouble(e -> e.getSal()));
        //Integer collect = employeeList.stream().mapToInt(Employee::getId).sum();
        System.out.println(collect); //1400000.0

        //averaging: taking average of all salaries
        Double average = employeeList.stream().collect(Collectors.averagingDouble(e -> e.getSal()));
        System.out.println(average); //350000.0

        //counting
        Long count = employeeList.stream().filter(e->e.getSal()>100000).collect(Collectors.counting());
        System.out.println(count); //3

        //joining
        String join = employeeList.stream().map(Employee::getName).collect(Collectors.joining("-","**","**"));
        //******************************************************************************************************************
        Optional<List<Employee>> opListEmp = Optional.of(employeeList);
        //System.out.println("---------------"+opListEmp);
        System.out.println(join);

        //Optional[[stream.basic.pojo.Employee@3d494fbf, stream.basic.pojo.Employee@1ddc4ec2, stream.basic.pojo.Employee@133314b, stream.basic.pojo.Employee@b1bc7ed]]

        //groupingBy
        Stream<Employee> stream = employeeList.stream();
        Map<String, List<Employee>> collect1 = stream.collect(Collectors.groupingBy(e -> e.getName(), Collectors.toList()));
        System.out.println(collect1);

        //reducing of Collectors class
        Comparator<Employee> empComparator = (e1, e2) -> e1.getId() - e2.getId();
        Optional<Employee> reducingOptinal = employeeList.stream().collect(Collectors.reducing(BinaryOperator.minBy(empComparator)));
        if (reducingOptinal.isPresent()) {
            System.out.println("Min Emp using reducing() method : " + reducingOptinal.get());
        }
    }
}
