package stream.rte;

import java.util.*;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Demo {

    public static void main(String[] args) {
        List<Employee> employeeList = new ArrayList<Employee>();

        employeeList.add(new Employee(111, "Jiya Brein", 32, "Female", "HR", 2011, 25000.0));
        employeeList.add(new Employee(122, "Paul Niksui", 25, "Male", "Sales And Marketing", 2015, 13500.0));
        employeeList.add(new Employee(133, "Martin Theron", 29, "Male", "Infrastructure", 2012, 18000.0));
        employeeList.add(new Employee(144, "Murali Gowda", 28, "Male", "Product Development", 2014, 32500.0));
        employeeList.add(new Employee(155, "Nima Roy", 27, "Female", "HR", 2013, 22700.0));
        employeeList.add(new Employee(166, "Iqbal Hussain", 43, "Male", "Security And Transport", 2016, 10500.0));
        employeeList.add(new Employee(177, "Manu Sharma", 35, "Male", "Account And Finance", 2010, 27000.0));
        employeeList.add(new Employee(188, "Wang Liu", 31, "Male", "Product Development", 2015, 34500.0));
        employeeList.add(new Employee(199, "Amelia Zoe", 24, "Female", "Sales And Marketing", 2016, 11500.0));
        employeeList.add(new Employee(200, "Jaden Dough", 38, "Male", "Security And Transport", 2015, 11000.5));
        employeeList.add(new Employee(211, "Jasna Kaur", 27, "Female", "Infrastructure", 2014, 15700.0));
        employeeList.add(new Employee(222, "Nitin Joshi", 25, "Male", "Product Development", 2016, 28200.0));
        employeeList.add(new Employee(233, "Jyothi Reddy", 27, "Female", "Account And Finance", 2013, 21300.0));
        employeeList.add(new Employee(244, "Nicolus Den", 24, "Male", "Sales And Marketing", 2017, 10700.5));
        employeeList.add(new Employee(255, "Ali Baig", 23, "Male", "Infrastructure", 2018, 12700.0));
        employeeList.add(new Employee(266, "Sanvi Pandey", 26, "Female", "Product Development", 2015, 28900.0));
        employeeList.add(new Employee(277, "Anuj Chettiar", 31, "Male", "Product Development", 2012, 35700.0));

        /**
         *   getGender
         */
        Map<String, Long> getNoOfMaleAndFemales = employeeList.stream().
                collect(Collectors.groupingBy(Employee::getGender, Collectors.counting()));
        //System.out.println(getNoOfMaleAndFemales); //{Male=11, Female=6}

        /**
         *  find all Departments
         */
        Stream<String> department = employeeList.stream().map(Employee::getDepartment).distinct();
        //department.forEach(System.out::println);

        /**
         *  get Highest Salary
         */
        Optional<Employee> highestPaidEmployee= employeeList.stream().max(Comparator.comparingDouble(Employee::getSalary));
        //System.out.println(highestPaidEmployee.get().getSalary());

        /**
         * grouping By Examples
         */
        Map<String, List<Employee>> collect1 = employeeList.stream().collect(Collectors.groupingBy(Employee::getDepartment));
        Map<Integer, List<Employee>> collect2 = employeeList.stream().collect(Collectors.groupingBy(z -> z.getAge()));
        Map<Integer, Long> collect = employeeList.stream().collect(Collectors.groupingBy(Employee::getAge, Collectors.counting()));
        //System.out.println(collect);

        /**
         *  Get Employee names who joined after 2012
         */
        List<String> empNames = employeeList.stream().filter(f -> f.getYearOfJoining() > 2015).map(Employee::getName).collect(Collectors.toList());
        //empNames.forEach(System.out::println);

        /**
         * Count the number of employees in each department
         */
        Map<String, Long> noOfEmpByDepart = employeeList.stream().collect(Collectors.groupingBy(Employee::getDepartment, Collectors.counting()));
        //noOfEmpByDepart.forEach((key, value) -> System.out.println(key + " : " + value));

        /**
         * Get the max salary of employee in each department
         */
        Map<String, Employee> maxSalaryByDepartment = employeeList.stream().collect(Collectors.groupingBy(Employee::getDepartment, Collectors.collectingAndThen(Collectors.maxBy(Comparator.comparing(Employee::getSalary)), Optional::get)));
        //maxSalaryByDepartment.forEach((s, employee) -> System.out.println(s+"--"+employee.getSalary()));



    }
}
